def test_bw_subcommand_deep_black(cli_runner, image_file):
    img_path = image_file("sample.png")
    ret = cli_runner("bw", "--filter", "DEEP_BLACK", str(img_path))
    assert ret.returncode == 0


def test_bw_subcommand_required_filter(cli_runner, image_file):
    img_path = image_file("sample.png")
    ret = cli_runner("bw", str(img_path))
    assert ret.returncode != 0


def test_bw_subcommand_all_filters(cli_runner, image_file, tmp_path):
    img_path = image_file("sample.png")
    output_dir = tmp_path / "output"

    ret = cli_runner(
        "bw",
        "--all-filters",
        "-od",
        str(output_dir),
        str(img_path),
    )

    assert ret.returncode == 0

    expected_files = [
        "bw_deep_black_sample.png",
        "bw_true_gray_sample.png",
        "bw_soft_light_sample.png",
    ]

    for filename in expected_files:
        assert (output_dir / filename).exists()



# Copyright (C) 2024,2025,2026 Kian-Meng Ang
#
# This program is free software: you can redistribute it and/or modify it under
# the terms of the GNU Affero General Public License as published by the Free
# Software Foundation, either version 3 of the License, or (at your option) any
# later version.
#
# This program is distributed in the hope that it will be useful, but WITHOUT
# ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS
# FOR A PARTICULAR PURPOSE. See the GNU Affero General Public License for more
# details.
#
# You should have received a copy of the GNU Affero General Public License
# along with this program. If not, see <https://www.gnu.org/licenses/>.

"""Border subcommand."""

from __future__ import annotations

import logging
from pathlib import Path
from typing import TYPE_CHECKING, Any

if TYPE_CHECKING:
    import argparse

from PIL import ImageColor, ImageOps

from fotolab import load_image, save_image

from .common import add_common_arguments, log_args_decorator

log = logging.getLogger(__name__)


def build_subparser(subparsers: argparse._SubParsersAction[Any]) -> None:
    """Build the subparser."""
    border_parser = subparsers.add_parser("border", help="add border to image")

    border_parser.set_defaults(func=run)

    add_common_arguments(border_parser)

    border_parser.add_argument(
        "-c",
        "--color",
        dest="color",
        type=str,
        default="black",
        help="set the color of border (default: '%(default)s')",
        metavar="COLOR",
    )

    border_parser.add_argument(
        "-w",
        "--width",
        dest="width",
        type=int,
        default=10,
        help="set the width of border in pixels (default: '%(default)s')",
        metavar="WIDTH",
    )

    border_parser.add_argument(
        "-wt",
        "--width-top",
        dest="width_top",
        type=int,
        default=0,
        help="set the width of top border in pixels (default: '%(default)s')",
        metavar="WIDTH",
    )

    border_parser.add_argument(
        "-wr",
        "--width-right",
        dest="width_right",
        type=int,
        default=0,
        help=(
            "set the width of right border in pixels (default: '%(default)s')"
        ),
        metavar="WIDTH",
    )

    border_parser.add_argument(
        "-wb",
        "--width-bottom",
        dest="width_bottom",
        type=int,
        default=0,
        help=(
            "set the width of bottom border in pixels (default: '%(default)s')"
        ),
        metavar="WIDTH",
    )

    border_parser.add_argument(
        "-wl",
        "--width-left",
        dest="width_left",
        type=int,
        default=0,
        help="set the width of left border in pixels (default: '%(default)s')",
        metavar="WIDTH",
    )


@log_args_decorator
def run(args: argparse.Namespace) -> None:
    """Run border subcommand.

    Args:
        args (argparse.Namespace): Config from command line arguments

    Returns:
        None
    """
    for image_filepath in [Path(f) for f in args.image_paths]:
        with load_image(image_filepath) as original_image:
            border = get_border(args)
            bordered_image = ImageOps.expand(
                original_image,
                border=border,
                fill=ImageColor.getrgb(args.color),
            )

            save_image(args, bordered_image, image_filepath, "border")


def get_border(
    args: argparse.Namespace,
) -> tuple[int, int, int, int]:
    """Calculate the border dimensions.

    Args:
        args (argparse.Namespace): Command line arguments

    Returns:
        Tuple[int, int, int, int]: Border dimensions in pixels as (left, top,
        right, bottom) widths. If individual widths are not specified,
        a uniform width is returned for all sides.
    """
    if any(
        [
            args.width_left,
            args.width_top,
            args.width_right,
            args.width_bottom,
        ],
    ):
        return (
            args.width_left,
            args.width_top,
            args.width_right,
            args.width_bottom,
        )
    # If no individual widths are specified, use the general width for all
    # sides
    return (args.width, args.width, args.width, args.width)



# Copyright (C) 2024,2025,2026 Kian-Meng Ang
#
# This program is free software: you can redistribute it and/or modify it under
# the terms of the GNU Affero General Public License as published by the Free
# Software Foundation, either version 3 of the License, or (at your option) any
# later version.
#
# This program is distributed in the hope that it will be useful, but WITHOUT
# ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS
# FOR A PARTICULAR PURPOSE. See the GNU Affero General Public License for more
# details.
#
# You should have received a copy of the GNU Affero General Public License
# along with this program. If not, see <https://www.gnu.org/licenses/>.

"""Resize subcommand."""

from __future__ import annotations

import logging
import math
from pathlib import Path
from typing import TYPE_CHECKING, Any

if TYPE_CHECKING:
    import argparse

from PIL import Image, ImageColor

from fotolab import load_image, save_image

from .common import add_common_arguments, log_args_decorator

log = logging.getLogger(__name__)


def _parse_aspect_ratio(ratio_str: str) -> float:
    """Parse an aspect ratio string (e.g., '16:9') into a float."""
    try:
        width, height = map(int, ratio_str.split(":"))
        if height == 0:
            msg = "error: height cannot be zero in aspect ratio."
            raise SystemExit(msg)
        return width / height
    except ValueError as e:
        msg = f"error: invalid aspect ratio format: {ratio_str}"
        raise SystemExit(msg) from e


DEFAULT_WIDTH = 600
DEFAULT_HEIGHT = 277


def build_subparser(subparsers: argparse._SubParsersAction[Any]) -> None:
    """Build the subparser."""
    resize_parser = subparsers.add_parser("resize", help="resize an image")

    resize_parser.set_defaults(func=run)

    add_common_arguments(resize_parser)

    resize_parser.add_argument(
        "-c",
        "--canvas",
        default=False,
        action="store_true",
        dest="canvas",
        help="paste image onto a larger canvas",
    )

    resize_parser.add_argument(
        "-l",
        "--canvas-color",
        default="black",
        dest="canvas_color",
        help=(
            "the color of the extended larger canvas(default: '%(default)s')"
        ),
    )

    # Define width and height arguments as optional with defaults.
    # The conditional logic (required/mutually exclusive) is now handled in the
    # run function.
    resize_parser.add_argument(
        "-W",
        "--width",
        dest="width",
        help="set the width of the image (default: '%(default)s')",
        type=int,
        default=DEFAULT_WIDTH,
        metavar="WIDTH",
    )

    resize_parser.add_argument(
        "-H",
        "--height",
        dest="height",
        help="set the height of the image (default: '%(default)s')",
        type=int,
        default=DEFAULT_HEIGHT,
        metavar="HEIGHT",
    )

    resize_parser.add_argument(
        "-ar",
        "--aspect-ratio",
        dest="aspect_ratio",
        help="set the aspect ratio of the image (e.g., '16:9', '4:3')",
        type=str,
        default=None,
        metavar="ASPECT_RATIO",
    )


@log_args_decorator
def run(args: argparse.Namespace) -> None:
    """Run resize subcommand.

    Args:
        args (argparse.Namespace): Config from command line arguments

    Returns:
        None
    """
    width_provided = args.width != DEFAULT_WIDTH
    height_provided = args.height != DEFAULT_HEIGHT
    ratio_provided = args.aspect_ratio is not None

    aspect_ratio = None

    if args.canvas:
        # Canvas mode: Both width and height are required
        if not (width_provided and height_provided):
            msg = (
                "error: argument -W/--width and -H/--height are required when "
                "using --canvas"
            )
            raise SystemExit(msg)
        if ratio_provided:
            log.warning(
                "warning: argument -ar/--aspect-ratio is ignored "
                "when using --canvas",
            )
    else:
        # Resize mode:
        if width_provided and height_provided:
            msg = (
                "error: argument -W/--width and -H/--height are mutually "
                "exclusive when not using --canvas"
            )
            raise SystemExit(msg)

        if ratio_provided:
            if not (width_provided or height_provided):
                msg = (
                    "error: argument -ar/--aspect-ratio requires either "
                    "-W/--width or -H/--height"
                )
                raise SystemExit(msg)
            aspect_ratio = _parse_aspect_ratio(args.aspect_ratio)

    for image_filepath in [Path(f) for f in args.image_paths]:
        with load_image(image_filepath) as original_image:
            if args.canvas:
                resized_image = _resize_image_onto_canvas(original_image, args)
            else:
                resized_image = _resize_image(
                    original_image,
                    args,
                    aspect_ratio,
                )
            save_image(args, resized_image, image_filepath, "resize")


def _resize_image_onto_canvas(
    original_image: Image.Image,
    args: argparse.Namespace,
) -> Image.Image:
    resized_image = Image.new(
        "RGB",
        (args.width, args.height),
        (*ImageColor.getrgb(args.canvas_color), 128),
    )
    x_offset = (args.width - original_image.width) // 2
    y_offset = (args.height - original_image.height) // 2
    resized_image.paste(original_image, (x_offset, y_offset))
    return resized_image


def _resize_image(
    original_image: Image.Image,
    args: argparse.Namespace,
    aspect_ratio: float | None,
) -> Image.Image:
    new_width, new_height = _calc_new_image_dimension(
        original_image,
        args,
        aspect_ratio,
    )
    return original_image.resize(
        (new_width, new_height),
        Image.Resampling.LANCZOS,
    )


def _calc_new_image_dimension(
    image: Image.Image,
    args: argparse.Namespace,
    aspect_ratio: float | None,
) -> tuple[int, int]:
    old_width, old_height = image.size
    log.debug("old image dimension: %d x %d", old_width, old_height)

    new_width = args.width
    new_height = args.height

    # Determine the aspect ratio to use for calculation
    if aspect_ratio is not None:
        ratio_to_use = aspect_ratio
        log.debug("using specified aspect ratio: %f", ratio_to_use)
    else:
        ratio_to_use = old_width / old_height
        log.debug("using original aspect ratio: %f", ratio_to_use)

    width_provided = new_width != DEFAULT_WIDTH
    height_provided = new_height != DEFAULT_HEIGHT

    match (width_provided, height_provided):
        case (True, False):
            # user provided width, calculate height
            new_height = math.ceil(new_width / ratio_to_use)
            log.debug("new height calculated based on width: %d", new_height)
        case (False, True):
            # user provided height, calculate width
            new_width = math.ceil(new_height * ratio_to_use)
            log.debug("new width calculated based on height: %d", new_width)
        case _:
            # if both are default, no calculation needed, use defaults
            # the case where both are non-default is disallowed by validation
            # in run()
            pass

    log.debug("new image dimension: %d x %d", new_width, new_height)
    return (new_width, new_height)



# Copyright (C) 2024,2025,2026 Kian-Meng Ang
#
# This program is free software: you can redistribute it and/or modify it under
# the terms of the GNU Affero General Public License as published by the Free
# Software Foundation, either version 3 of the License, or (at your option) any
# later version.
#
# This program is distributed in the hope that it will be useful, but WITHOUT
# ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS
# FOR A PARTICULAR PURPOSE. See the GNU Affero General Public License for more
# details.
#
# You should have received a copy of the GNU Affero General Public License
# along with this program. If not, see <https://www.gnu.org/licenses/>.

"""Contrast subcommand."""

from __future__ import annotations

import argparse
import logging
from pathlib import Path
from typing import Any

from PIL import ImageOps

from fotolab import load_image, save_image

from .common import add_common_arguments, log_args_decorator

log = logging.getLogger(__name__)


# pylint: disable=protected-access
MAX_CUTOFF: int = 50


def _validate_cutoff(value: str) -> float:
    """Validate the cutoff value."""
    try:
        f_value = float(value)
    except ValueError as e:
        msg = f"invalid float value: '{value}'"
        raise argparse.ArgumentTypeError(msg) from e
    if not 0 <= f_value <= MAX_CUTOFF:
        msg = f"cutoff value {f_value} must be between 0 and 50"
        raise argparse.ArgumentTypeError(msg)
    return f_value


def build_subparser(subparsers: argparse._SubParsersAction[Any]) -> None:
    """Build the subparser."""
    contrast_parser = subparsers.add_parser(
        "contrast",
        help="contrast an image.",
    )

    contrast_parser.set_defaults(func=run)

    add_common_arguments(contrast_parser)

    contrast_parser.add_argument(
        "-c",
        "--cutoff",
        dest="cutoff",
        type=_validate_cutoff,
        default=1.0,
        help=(
            "set the percentage (0-50) of lightest or darkest pixels"
            " to discard from histogram"
            " (default: '%(default)s')"
        ),
        metavar="CUTOFF",
    )


@log_args_decorator
def run(args: argparse.Namespace) -> None:
    """Run contrast subcommand.

    Args:
        args (argparse.Namespace): Config from command line arguments

    Returns:
        None
    """
    for image_path_str in args.image_paths:
        with load_image(Path(image_path_str)) as original_image:
            if original_image.mode == "RGBA":
                # Split the image into RGB and Alpha channels
                rgb_image = original_image.convert("RGB")
                alpha_channel = original_image.getchannel("A")

                # Apply autocontrast to the RGB part
                contrasted_rgb = ImageOps.autocontrast(
                    rgb_image,
                    cutoff=args.cutoff,
                )

                # Merge the contrasted RGB part with the original Alpha channel
                contrasted_rgb.putalpha(alpha_channel)
                contrast_image = contrasted_rgb
            else:
                # For other modes (like RGB, L, etc.), apply autocontrast
                # directly
                contrast_image = ImageOps.autocontrast(
                    original_image,
                    cutoff=args.cutoff,
                )

            save_image(args, contrast_image, Path(image_path_str), "contrast")



# Copyright (C) 2024,2025,2026 Kian-Meng Ang
#
# This program is free software: you can redistribute it and/or modify it under
# the terms of the GNU Affero General Public License as published by the Free
# Software Foundation, either version 3 of the License, or (at your option) any
# later version.
#
# This program is distributed in the hope that it will be useful, but WITHOUT
# ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS
# FOR A PARTICULAR PURPOSE. See the GNU Affero General Public License for more
# details.
#
# You should have received a copy of the GNU Affero General Public License
# along with this program. If not, see <https://www.gnu.org/licenses/>.

"""Sharpen subcommand."""

from __future__ import annotations

import logging
from pathlib import Path
from typing import TYPE_CHECKING, Any

if TYPE_CHECKING:
    import argparse

from PIL import ImageFilter

from fotolab import load_image, save_gif_image, save_image

from .common import add_common_arguments, log_args_decorator

log = logging.getLogger(__name__)


def build_subparser(subparsers: argparse._SubParsersAction[Any]) -> None:
    """Build the subparser."""
    sharpen_parser = subparsers.add_parser("sharpen", help="sharpen an image")

    sharpen_parser.set_defaults(func=run)

    add_common_arguments(sharpen_parser)

    sharpen_parser.add_argument(
        "-r",
        "--radius",
        dest="radius",
        type=int,
        default=1,
        help="set the radius or size of edges (default: '%(default)s')",
        metavar="RADIUS",
    )

    sharpen_parser.add_argument(
        "-p",
        "--percent",
        dest="percent",
        type=int,
        default=100,
        help=(
            "set the amount of overall strength of sharpening effect "
            "(default: '%(default)s')"
        ),
        metavar="PERCENT",
    )

    sharpen_parser.add_argument(
        "-t",
        "--threshold",
        dest="threshold",
        type=int,
        default=3,
        help=(
            "set the minimum brightness changed to be sharpened "
            "(default: '%(default)s')"
        ),
        metavar="THRESHOLD",
    )

    sharpen_parser.add_argument(
        "-ba",
        "--before-after",
        default=False,
        action="store_true",
        dest="before_after",
        help="generate a GIF showing before and after changes",
    )


@log_args_decorator
def run(args: argparse.Namespace) -> None:
    """Run sharpen subcommand.

    Args:
        args (argparse.Namespace): Config from command line arguments

    Returns:
        None
    """
    for image_path_str in args.image_paths:
        with load_image(Path(image_path_str)) as original_image:
            sharpen_image = original_image.filter(
                ImageFilter.UnsharpMask(
                    args.radius,
                    percent=args.percent,
                    threshold=args.threshold,
                ),
            )
            if args.before_after:
                save_gif_image(
                    args,
                    Path(image_path_str),
                    original_image,
                    sharpen_image,
                    "sharpen",
                )
            else:
                save_image(
                    args,
                    sharpen_image,
                    Path(image_path_str),
                    "sharpen",
                )



# Copyright (C) 2024,2025,2026 Kian-Meng Ang
#
# This program is free software: you can redistribute it and/or modify it under
# the terms of the GNU Affero General Public License as published by the Free
# Software Foundation, either version 3 of the License, or (at your option) any
# later version.
#
# This program is distributed in the hope that it will be useful, but WITHOUT
# ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS
# FOR A PARTICULAR PURPOSE. See the GNU Affero General Public License for more
# details.
#
# You should have received a copy of the GNU Affero General Public License
# along with this program. If not, see <https://www.gnu.org/licenses/>.

"""A console program that manipulate images."""

import argparse
import logging
import os
import shutil
import subprocess
import sys
from collections.abc import Iterator
from contextlib import contextmanager
from importlib import metadata
from pathlib import Path
from typing import Any

from PIL import Image

__version__ = metadata.version("fotolab")

log = logging.getLogger(__name__)


def save_gif_image(
    args: argparse.Namespace,
    image_filepath: Path,
    original_image: Image.Image,
    after_image: Image.Image,
    subcommand: str,
) -> None:
    """Save the original and after image."""
    gif_kwargs = {
        "format": "gif",
        "append_images": [after_image],
        "save_all": True,
        "duration": 2500,
        "loop": 0,
        "optimize": True,
    }
    save_image(args, original_image, image_filepath, subcommand, **gif_kwargs)


def save_image(
    args: argparse.Namespace,
    image: Image.Image,
    output_filepath: Path,
    subcommand: str,
    **kwargs: Any,
) -> None:
    """Save image with additional options and handle opening.

    Args:
        args (argparse.Namespace): Config from command line arguments.
        image (Image.Image): The image to save.
        output_filepath (Path): The path to save the image to.
        subcommand (str): The name of the subcommand.
        **kwargs: Additional keyword arguments for Image.save().
    """
    new_filename = _get_output_filename(args, output_filepath, subcommand)
    log.info("%s image: %s", subcommand, new_filename.resolve())
    image.save(new_filename, **kwargs)

    if args.open:
        open_image(new_filename)


def _get_output_filename(
    args: argparse.Namespace,
    image_file: Path,
    subcommand: str,
) -> Path:
    """Build and return output filename based on the command line options."""
    if args.overwrite:
        return image_file.with_name(image_file.name)

    output_dir = Path(args.output_dir)
    output_dir.mkdir(parents=True, exist_ok=True)

    if args.output_filename:
        new_filename = args.output_filename
    else:
        new_filename = f"{subcommand}_{image_file.stem}{image_file.suffix}"
    return Path(output_dir / new_filename)


def open_image(filename: Path) -> None:
    """Open generated image using default program."""
    try:
        if sys.platform == "linux":
            executable = shutil.which("xdg-open")
            if executable:
                subprocess.call([executable, filename])
        elif sys.platform == "darwin":
            executable = shutil.which("open")
            if executable:
                subprocess.call([executable, filename])
        elif sys.platform == "win32":
            os.startfile(filename)
        log.info("open image: %s", filename.resolve())

    except (OSError, FileNotFoundError):
        log.exception("Error opening image: %s", filename)


@contextmanager
def load_image(filename: Path) -> Iterator[Image.Image]:
    """Load image using a context manager to ensure file handle is closed."""
    image = None
    try:
        image = Image.open(filename)
        yield image
    finally:
        if image is not None:
            image.close()



# Copyright (C) 2024,2025,2026 Kian-Meng Ang

# This program is free software: you can redistribute it and/or modify it under
# the terms of the GNU Affero General Public License as published by the Free
# Software Foundation, either version 3 of the License, or (at your option) any
# later version.
#
# This program is distributed in the hope that it will be useful, but WITHOUT
# ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS
# FOR A PARTICULAR PURPOSE. See the GNU Affero General Public License for more
# details.
#
# You should have received a copy of the GNU Affero General Public License
# along with this program. If not, see <https://www.gnu.org/licenses/>.


def test_border_subcommand(cli_runner, image_file):
    img_path = image_file("sample.png")
    ret = cli_runner("border", str(img_path), "--width", "10")
    assert ret.returncode == 0


def test_border_subcommand_with_individual_widths(cli_runner, image_file):
    img_path = image_file("sample.png")
    ret = cli_runner(
        "border",
        str(img_path),
        "--width-top",
        "5",
        "--width-right",
        "10",
        "--width-bottom",
        "15",
        "--width-left",
        "20",
    )
    assert ret.returncode == 0



# Copyright (C) 2024,2025,2026 Kian-Meng Ang
#
# This program is free software: you can redistribute it and/or modify it under
# the terms of the GNU Affero General Public License as published by the Free
# Software Foundation, either version 3 of the License, or (at your option) any
# later version.
#
# This program is distributed in the hope that it will be useful, but WITHOUT
# ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS
# FOR A PARTICULAR PURPOSE. See the GNU Affero General Public License for more
# details.
#
# You should have received a copy of the GNU Affero General Public License
# along with this program. If not, see <https://www.gnu.org/licenses/>.

"""Crop subcommand."""

from __future__ import annotations

import logging
from pathlib import Path
from typing import TYPE_CHECKING, Any

if TYPE_CHECKING:
    import argparse

from fotolab import load_image, save_image

from .common import add_common_arguments, log_args_decorator

log = logging.getLogger(__name__)


def build_subparser(subparsers: argparse._SubParsersAction[Any]) -> None:
    """Build the subparser."""
    crop_parser = subparsers.add_parser("crop", help="crop an image")

    crop_parser.set_defaults(func=run)

    add_common_arguments(crop_parser)

    crop_parser.add_argument(
        "-b",
        "--box",
        dest="box",
        help=(
            "set the crop area as a 4-tuple (left, upper, right, lower), "
            "e.g., '100,100,500,500'"
        ),
        type=str,
        required=True,
        metavar="BOX",
    )


EXPECTED_PARTS: int = 4


def _parse_box(box_str: str) -> tuple[int, int, int, int]:
    """Parse a box string into a 4-tuple of integers."""
    try:
        parts = list(map(int, box_str.split(",")))
    except ValueError as e:
        msg = f"error: invalid box format: {box_str}. {e}"
        raise SystemExit(msg) from e

    if len(parts) != EXPECTED_PARTS:
        msg = (
            f"error: invalid box format: {box_str}. "
            "Box must contain exactly four comma-separated integers."
        )
        raise SystemExit(msg)

    left, upper, right, lower = parts
    if left < 0 or upper < 0 or right < 0 or lower < 0:
        msg = (
            f"error: invalid box format: {box_str}. "
            "All coordinates must be non-negative."
        )
        raise SystemExit(msg)

    if left >= right or upper >= lower:
        msg = (
            f"error: invalid box format: {box_str}. "
            "Left coordinate must be less than right, "
            "and upper must be less than lower."
        )
        raise SystemExit(msg)

    return (left, upper, right, lower)


@log_args_decorator
def run(args: argparse.Namespace) -> None:
    """Run crop subcommand.

    Args:
        args (argparse.Namespace): Config from command line arguments

    Returns:
        None
    """
    crop_box = _parse_box(args.box)

    for image_filepath in [Path(f) for f in args.image_paths]:
        with load_image(image_filepath) as original_image:
            log.debug("Original image size: %s", original_image.size)

            # Check if the crop box is within the image boundaries
            img_width, img_height = original_image.size
            left, upper, right, lower = crop_box

            if right > img_width or lower > img_height:
                log.warning(
                    "Crop box (%s) extends beyond image boundaries (%d, %d). "
                    "Cropping to the maximum possible area.",
                    args.box,
                    img_width,
                    img_height,
                )
                # Adjust the box to fit within the image
                right = min(right, img_width)
                lower = min(lower, img_height)
                crop_box = (left, upper, right, lower)

            if left >= right or upper >= lower:
                log.error(
                    "Adjusted crop box (%s) is invalid or results in "
                    "zero area. Skipping image.",
                    crop_box,
                )
                continue

            cropped_image = original_image.crop(crop_box)
            log.debug("Cropped image size: %s", cropped_image.size)

            save_image(args, cropped_image, image_filepath, "crop")



# Copyright (C) 2024,2025,2026 Kian-Meng Ang

# This program is free software: you can redistribute it and/or modify it under
# the terms of the GNU Affero General Public License as published by the Free
# Software Foundation, either version 3 of the License, or (at your option) any
# later version.
#
# This program is distributed in the hope that it will be useful, but WITHOUT
# ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS
# FOR A PARTICULAR PURPOSE. See the GNU Affero General Public License for more
# details.
#
# You should have received a copy of the GNU Affero General Public License
# along with this program. If not, see <https://www.gnu.org/licenses/>.

from pathlib import Path

from PIL import Image


def test_watermark_subcommand(cli_runner, image_file):
    img_path = image_file("sample.png")
    ret = cli_runner("watermark", str(img_path), "--text", "Test")
    assert ret.returncode == 0


def test_watermark_color_mode_rgb_to_rgba_to_rgb_conversion(
    cli_runner,
    image_file,
):
    img_path = image_file("rgb_sample.jpg")

    output_dir = Path("output")
    ret = cli_runner(
        "watermark",
        str(img_path),
        "--output-dir",
        str(output_dir),
        "--text",
        "Test",
    )
    assert ret.returncode == 0

    output_filename = (
        Path("scripttest")
        / output_dir
        / f"watermark_{img_path.stem}{img_path.suffix}"
    )

    with Image.open(output_filename) as image:
        assert image.mode == "RGB"
